from typing import Optional, Tuple

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .Section import Section
from .TransverseShearBeam import TransverseShearBeam
from ..UtilityAndView.SymbolicConstant import abaqusConstants as C
from ..UtilityAndView.abaqusConstants import (ANALYSIS_DEFAULT, Boolean, CONSTANT,
                                              FULLY, LINEAR,
                                              OFF)


@abaqus_class_doc
class BeamSection(Section):
    """The BeamSection object defines the properties of a beam section.
    The BeamSection object is derived from the Section object.

    .. note:: 
        This object can be accessed by::

            import section
            mdb.models[name].sections[name]
            import odbSection
            session.odbs[name].sections[name]

        The table data for this object are:
        
        - E, the Young's modulus of the section.
        - G, the torsional shear modulus of the section.
        - Thermal expansion coefficient, if using thermal expansion.
        - Temperature, if the data depend on temperature.
        - Value of the first field variable, if the data depend on field variables.
        - Value of the second field variable.
        - Etc.

        The corresponding analysis keywords are:

        - BEAM GENERAL SECTION
        - BEAM SECTION
        - BEAM FLUID INERTIA
        - CENTROID
        - DAMPING
        - SHEAR CENTER
        - SECTION POINTS
    """

    #: A :py:class:`~abaqus.Section.TransverseShearBeam.TransverseShearBeam` object specifying the transverse shear stiffness properties.
    beamTransverseShear: TransverseShearBeam = TransverseShearBeam(ANALYSIS_DEFAULT)

    #: A String specifying the repository key.
    name: str

    #: A SymbolicConstant specifying the integration method for the section. Possible values
    #: are BEFORE_ANALYSIS and DURING_ANALYSIS.
    integration: Literal[C.BEFORE_ANALYSIS, C.DURING_ANALYSIS]

    #: A String specifying the name of the profile. This argument represents the start profile
    #: in case of **beamShape** = TAPERED.
    profile: str

    #: A Float specifying the Poisson's ratio of the section. The default value is 0.0.
    poissonRatio: float = 0

    #: A Boolean specifying whether to use thermal expansion data. The default value is OFF.
    thermalExpansion: Boolean = OFF

    #: A Boolean specifying whether the data depend on temperature. The default value is OFF.
    temperatureDependency: Boolean = OFF

    #: An Int specifying the number of field variable dependencies. The default value is 0.
    dependencies: int = 0

    #: None or a Float specifying the density of the section. The default value is None.
    density: Optional[float] = None

    #: None or a Float specifying the reference temperature of the section. The default value
    #: is None.
    referenceTemperature: Optional[float] = None

    #: A SymbolicConstant specifying the temperature variation for the section. Possible values
    #: are LINEAR and INTERPOLATED. The default value is LINEAR.
    temperatureVar: Literal[C.LINEAR, C.INTERPOLATED] = LINEAR

    #: A Float specifying the αRαR factor to create mass proportional damping in
    #: direct-integration dynamics. The default value is 0.0.
    alphaDamping: float = 0

    #: A Float specifying the βRβR factor to create stiffness proportional damping in
    #: direct-integration dynamics. The default value is 0.0.
    betaDamping: float = 0

    #: A Float specifying the fraction of critical damping to be used in calculating composite
    #: damping factors for the modes (for use in modal dynamics). The default value is 0.0.
    compositeDamping: float = 0

    #: A Boolean specifying whether added mass effects will be simulated. The default value is
    #: OFF.
    useFluidInertia: Boolean = OFF

    #: A SymbolicConstant specifying whether the section is either full submerged or half
    #: submerged. This argument applies only when **useFluidInertia** = True. Possible values are
    #: FULLY and HALF. The default value is FULLY.
    submerged: Literal[C.FULLY, C.HALF] = FULLY

    #: None or a Float specifying the mass density of the fluid. This argument applies only
    #: when **useFluidInertia** = True and must be specified in that case. The default value is
    #: None.
    fluidMassDensity: Optional[float] = None

    #: None or a Float specifying the radius of the cylindrical cross-section. This argument
    #: applies only when **useFluidInertia** = True and must be specified in that case. The
    #: default value is None.
    crossSectionRadius: Optional[float] = None

    #: A Float specifying the added mass coefficient, CACA, for lateral motions of the beam.
    #: This argument applies only when*useFluidInertia* = True. The default value is 1.0.
    lateralMassCoef: float = 1

    #: A Float specifying the added mass coefficient, C(A−E)C(A-E), for motions along the axis
    #: of the beam. This argument affects only the term added to the free end(s) of the beam,
    #: and applies only when **useFluidInertia** = True. The default value is 0.0.
    axialMassCoef: float = 0

    #: A Float specifying the local 1-coordinate of the center of the cylindrical cross-section
    #: with respect to the beam cross-section. This argument applies only when
    #: **useFluidInertia** = True. The default value is 0.0.
    massOffsetX: float = 0

    #: A Float specifying the local 2-coordinate of the center of the cylindrical cross-section
    #: with respect to the beam cross-section. This argument applies only when
    #: **useFluidInertia** = True. The default value is 0.0.
    massOffsetY: float = 0

    #: A SymbolicConstant specifying the change in cross-section of the beam along length.
    #: Possible values are CONSTANT and TAPERED. The default value is CONSTANT. This parameter
    #: is available for manipulating the model database but not for the ODB API.
    beamShape: Literal[C.CONSTANT, C.TAPERED] = CONSTANT

    #: A String specifying the name of the material. The default value is an empty string. The
    #: material is required when **integration** is "DURING_ANALYSIS".
    material: str = ""

    #: A sequence of sequences of Floats specifying the items described below. The default
    #: value is an empty sequence.
    table: tuple = ()

    #: A sequence of pairs of Floats specifying the positions at which output is requested. The
    #: default value is an empty sequence.
    outputPts: tuple = ()

    #: A pair of Floats specifying the **X - Y** coordinates of the centroid. The default value is
    #: (0.0, 0.0).
    centroid: Tuple[float, float] = (0.0, 0.0)

    #: A pair of Floats specifying the **X - Y** coordinates of the shear center. The default value
    #: is (0.0, 0.0).
    shearCenter: Tuple[float, float] = (0.0, 0.0)

    #: A String specifying the name of the end profile. The type of the end profile must be
    #: same as that of the start profile. This argument is valid only when **beamShape** = TAPERED.
    #: The default value is an empty string. This parameter is available for manipulating the
    #: model database but not for the ODB API.
    profileEnd: str = ""

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        integration: Literal[C.BEFORE_ANALYSIS, C.DURING_ANALYSIS],
        profile: str,
        poissonRatio: float = 0,
        thermalExpansion: Boolean = OFF,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        density: Optional[float] = None,
        referenceTemperature: Optional[float] = None,
        temperatureVar: Literal[C.LINEAR, C.INTERPOLATED] = LINEAR,
        alphaDamping: float = 0,
        betaDamping: float = 0,
        compositeDamping: float = 0,
        useFluidInertia: Boolean = OFF,
        submerged: Literal[C.FULLY, C.HALF] = FULLY,
        fluidMassDensity: Optional[float] = None,
        crossSectionRadius: Optional[float] = None,
        lateralMassCoef: float = 1,
        axialMassCoef: float = 0,
        massOffsetX: float = 0,
        massOffsetY: float = 0,
        beamShape: Literal[C.CONSTANT, C.TAPERED] = CONSTANT,
        material: str = "",
        table: tuple = (),
        outputPts: tuple = (),
        centroid: Tuple[float, float] = (0.0, 0.0),
        shearCenter: Tuple[float, float] = (0.0, 0.0),
        profileEnd: str = "",
    ) -> None:
        """This method creates a BeamSection object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].BeamSection
                session.odbs[name].BeamSection

        Parameters
        ----------
        name
            A String specifying the repository key.
        integration
            A SymbolicConstant specifying the integration method for the section. Possible values
            are BEFORE_ANALYSIS and DURING_ANALYSIS.
        profile
            A String specifying the name of the profile. This argument represents the start profile
            in case of **beamShape** = TAPERED.
        poissonRatio
            A Float specifying the Poisson's ratio of the section. The default value is 0.0.
        thermalExpansion
            A Boolean specifying whether to use thermal expansion data. The default value is OFF.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.
        density
            None or a Float specifying the density of the section. The default value is None.
        referenceTemperature
            None or a Float specifying the reference temperature of the section. The default value
            is None.
        temperatureVar
            A SymbolicConstant specifying the temperature variation for the section. Possible values
            are LINEAR and INTERPOLATED. The default value is LINEAR.
        alphaDamping
            A Float specifying the αRαR factor to create mass proportional damping in
            direct-integration dynamics. The default value is 0.0.
        betaDamping
            A Float specifying the βRβR factor to create stiffness proportional damping in
            direct-integration dynamics. The default value is 0.0.
        compositeDamping
            A Float specifying the fraction of critical damping to be used in calculating composite
            damping factors for the modes (for use in modal dynamics). The default value is 0.0.
        useFluidInertia
            A Boolean specifying whether added mass effects will be simulated. The default value is
            OFF.
        submerged
            A SymbolicConstant specifying whether the section is either full submerged or half
            submerged. This argument applies only when **useFluidInertia** = True. Possible values are
            FULLY and HALF. The default value is FULLY.
        fluidMassDensity
            None or a Float specifying the mass density of the fluid. This argument applies only
            when **useFluidInertia** = True and must be specified in that case. The default value is
            None.
        crossSectionRadius
            None or a Float specifying the radius of the cylindrical cross-section. This argument
            applies only when **useFluidInertia** = True and must be specified in that case. The
            default value is None.
        lateralMassCoef
            A Float specifying the added mass coefficient, CACA, for lateral motions of the beam.
            This argument applies only when*useFluidInertia* = True. The default value is 1.0.
        axialMassCoef
            A Float specifying the added mass coefficient, C(A−E)C(A-E), for motions along the axis
            of the beam. This argument affects only the term added to the free end(s) of the beam,
            and applies only when **useFluidInertia** = True. The default value is 0.0.
        massOffsetX
            A Float specifying the local 1-coordinate of the center of the cylindrical cross-section
            with respect to the beam cross-section. This argument applies only when
            **useFluidInertia** = True. The default value is 0.0.
        massOffsetY
            A Float specifying the local 2-coordinate of the center of the cylindrical cross-section
            with respect to the beam cross-section. This argument applies only when
            **useFluidInertia** = True. The default value is 0.0.
        beamShape
            A SymbolicConstant specifying the change in cross-section of the beam along length.
            Possible values are CONSTANT and TAPERED. The default value is CONSTANT. This parameter
            is available for manipulating the model database but not for the ODB API.
        material
            A String specifying the name of the material. The default value is an empty string. The
            material is required when **integration** is "DURING_ANALYSIS".
        table
            A sequence of sequences of Floats specifying the items described below. The default
            value is an empty sequence.
        outputPts
            A sequence of pairs of Floats specifying the positions at which output is requested. The
            default value is an empty sequence.
        centroid
            A pair of Floats specifying the **X - Y** coordinates of the centroid. The default value is
            (0.0, 0.0).
        shearCenter
            A pair of Floats specifying the **X - Y** coordinates of the shear center. The default value
            is (0.0, 0.0).
        profileEnd
            A String specifying the name of the end profile. The type of the end profile must be
            same as that of the start profile. This argument is valid only when **beamShape** = TAPERED.
            The default value is an empty string. This parameter is available for manipulating the
            model database but not for the ODB API.

        Returns
        -------
        BeamSection
            A :py:class:`~abaqus.Section.BeamSection.BeamSection` object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        poissonRatio: float = 0,
        thermalExpansion: Boolean = OFF,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        density: Optional[float] = None,
        referenceTemperature: Optional[float] = None,
        temperatureVar: Literal[C.LINEAR, C.INTERPOLATED] = LINEAR,
        alphaDamping: float = 0,
        betaDamping: float = 0,
        compositeDamping: float = 0,
        useFluidInertia: Boolean = OFF,
        submerged: Literal[C.FULLY, C.HALF] = FULLY,
        fluidMassDensity: Optional[float] = None,
        crossSectionRadius: Optional[float] = None,
        lateralMassCoef: float = 1,
        axialMassCoef: float = 0,
        massOffsetX: float = 0,
        massOffsetY: float = 0,
        beamShape: Literal[C.CONSTANT, C.TAPERED] = CONSTANT,
        material: str = "",
        table: tuple = (),
        outputPts: tuple = (),
        centroid: Tuple[float, float] = (0.0, 0.0),
        shearCenter: Tuple[float, float] = (0.0, 0.0),
        profileEnd: str = "",
    ) -> None:
        """This method modifies the BeamSection object.

        Parameters
        ----------
        poissonRatio
            A Float specifying the Poisson's ratio of the section. The default value is 0.0.
        thermalExpansion
            A Boolean specifying whether to use thermal expansion data. The default value is OFF.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.
        density
            None or a Float specifying the density of the section. The default value is None.
        referenceTemperature
            None or a Float specifying the reference temperature of the section. The default value
            is None.
        temperatureVar
            A SymbolicConstant specifying the temperature variation for the section. Possible values
            are LINEAR and INTERPOLATED. The default value is LINEAR.
        alphaDamping
            A Float specifying the αRαR factor to create mass proportional damping in
            direct-integration dynamics. The default value is 0.0.
        betaDamping
            A Float specifying the βRβR factor to create stiffness proportional damping in
            direct-integration dynamics. The default value is 0.0.
        compositeDamping
            A Float specifying the fraction of critical damping to be used in calculating composite
            damping factors for the modes (for use in modal dynamics). The default value is 0.0.
        useFluidInertia
            A Boolean specifying whether added mass effects will be simulated. The default value is
            OFF.
        submerged
            A SymbolicConstant specifying whether the section is either full submerged or half
            submerged. This argument applies only when **useFluidInertia** = True. Possible values are
            FULLY and HALF. The default value is FULLY.
        fluidMassDensity
            None or a Float specifying the mass density of the fluid. This argument applies only
            when **useFluidInertia** = True and must be specified in that case. The default value is
            None.
        crossSectionRadius
            None or a Float specifying the radius of the cylindrical cross-section. This argument
            applies only when **useFluidInertia** = True and must be specified in that case. The
            default value is None.
        lateralMassCoef
            A Float specifying the added mass coefficient, CACA, for lateral motions of the beam.
            This argument applies only when*useFluidInertia* = True. The default value is 1.0.
        axialMassCoef
            A Float specifying the added mass coefficient, C(A−E)C(A-E), for motions along the axis
            of the beam. This argument affects only the term added to the free end(s) of the beam,
            and applies only when **useFluidInertia** = True. The default value is 0.0.
        massOffsetX
            A Float specifying the local 1-coordinate of the center of the cylindrical cross-section
            with respect to the beam cross-section. This argument applies only when
            **useFluidInertia** = True. The default value is 0.0.
        massOffsetY
            A Float specifying the local 2-coordinate of the center of the cylindrical cross-section
            with respect to the beam cross-section. This argument applies only when
            **useFluidInertia** = True. The default value is 0.0.
        beamShape
            A SymbolicConstant specifying the change in cross-section of the beam along length.
            Possible values are CONSTANT and TAPERED. The default value is CONSTANT. This parameter
            is available for manipulating the model database but not for the ODB API.
        material
            A String specifying the name of the material. The default value is an empty string. The
            material is required when **integration** is "DURING_ANALYSIS".
        table
            A sequence of sequences of Floats specifying the items described below. The default
            value is an empty sequence.
        outputPts
            A sequence of pairs of Floats specifying the positions at which output is requested. The
            default value is an empty sequence.
        centroid
            A pair of Floats specifying the **X - Y** coordinates of the centroid. The default value is
            (0.0, 0.0).
        shearCenter
            A pair of Floats specifying the **X - Y** coordinates of the shear center. The default value
            is (0.0, 0.0).
        profileEnd
            A String specifying the name of the end profile. The type of the end profile must be
            same as that of the start profile. This argument is valid only when **beamShape** = TAPERED.
            The default value is an empty string. This parameter is available for manipulating the
            model database but not for the ODB API.
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

@abaqus_class_doc
class TieConstraint:
    """The TieConstraint object.

    .. note:: 
        This object can be accessed by::

            import visualization
            session.odbData[name].ties[i]
    """

    #: A String specifying the tie constraint name. This attribute is read-only.
    name: str = ""

    @abaqus_method_doc
    def constraintData(self):
        """This method returns constraint data if any are associated with the object

        Returns
        -------
        Tuple[float, ...]
            A tuple containing coordinates of the nodes pertaining to the constraint.
        """
        # TODO: Implement this method
        ...



from typing import Dict

from abqpy.decorators import abaqus_class_doc
from .OdbDiagnosticAttempt import OdbDiagnosticAttempt
from ..UtilityAndView.abaqusConstants import Boolean, OFF


@abaqus_class_doc
class OdbDiagnosticIncrement:
    """The OdbDiagnosticIncrement object.

    .. note:: 
        This object can be accessed by::

            import visualization
            session.odbData[name].diagnosticData.steps[i].increments[i]
    """

    #: A repository of OdbDiagnosticAttempt objects.
    attempts: Dict[str, OdbDiagnosticAttempt] = {}

    #: A float specifying the size of the initial increment. This attribute is read-only.
    initialSize: str = ""

    #: A boolean specifying whether the solution converged for the particular increment. This
    #: attribute is read-only.
    isConverged: Boolean = OFF

    #: An int specifying the number of attempts for the particular increment. This attribute is
    #: read-only.
    numberOfAttempts: str = ""

    #: An int specifying the number of element diagnostics encountered for the particular
    #: increment. This attribute is read-only.
    numberOfElementDiagnostics: str = ""

    #: A float specifying the size of the particular increment. This attribute is read-only.
    size: str = ""

    #: A float specifying the amount of step time completed in the particular increment. This
    #: attribute is read-only.
    stepTimeCompleted: str = ""



from typing import Optional, overload

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.SymbolicConstant import abaqusConstants as C
from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class HistoryOutput:
    """The HistoryOutput object contains the history output at a point for the specified
    variable.

    .. note:: 
        This object can be accessed by::

            import odbAccess
            session.odbs[name].steps[name].historyRegions[name].historyOutputs[name]
    """

    #: A tuple of pairs of Floats specifying the pairs (*frameValue*, **value**) where
    #: **frameValue** is either time, frequency, or mode and **value** is the value of the
    #: specified variable at **frameValue**. (This value depends on the type of the variable.)
    data: Optional[float] = None

    #: A tuple of pairs of Floats specifying the imaginary portion of a specified complex
    #: variable at each frame value (time, frequency, or mode). The pairs have the form
    #: (*frameValue*, **value**).
    conjugateData: Optional[float] = None

    #: A String specifying the output variable name.
    name: str

    #: A String specifying the output variable.
    description: str

    #: A SymbolicConstant specifying the output type. Only SCALAR is currently supported.
    type: SymbolicConstant

    #: A sequence of SymbolicConstants specifying which invariants should be calculated for
    #: this field. Possible values are MAGNITUDE, MISES, TRESCA, PRESS, INV3, MAX_PRINCIPAL,
    #: MID_PRINCIPAL, and MIN_PRINCIPAL. The default value is an empty sequence.
    validInvariants: Optional[SymbolicConstant] = None

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        description: str,
        type: Literal[C.SCALAR],
        validInvariants: Optional[SymbolicConstant] = None,
    ) -> None:
        """This method creates a HistoryOutput object.

        .. note:: 
            This function can be accessed by::

                session.odbs[name].steps[name].historyRegions[name].HistoryOutput

        Parameters
        ----------
        name
            A String specifying the output variable name.
        description
            A String specifying the output variable.
        type
            A SymbolicConstant specifying the output type. Only SCALAR is currently supported.
        validInvariants
            A sequence of SymbolicConstants specifying which invariants should be calculated for
            this field. Possible values are MAGNITUDE, MISES, TRESCA, PRESS, INV3, MAX_PRINCIPAL,
            MID_PRINCIPAL, and MIN_PRINCIPAL. The default value is an empty sequence.

        Returns
        -------
        HistoryOutput
            A :py:class:`~abaqus.Odb.HistoryOutput.HistoryOutput` object.
        """
        ...

    @overload
    @abaqus_method_doc
    def addData(self, frame: str, value: str) -> None:
        """This method adds data to the **data** member of the HistoryOutput object.

        Parameters
        ----------
        frame
            A Double specifying the frame value. **frame** can be specified in step time, frequency,
            or mode number.
        value
            A Double specifying the value of the variable at the frame value specified in **frame**.
        """
        ...

    @overload
    @abaqus_method_doc
    def addData(self, frame: tuple, value: tuple) -> None:
        """This method adds data to the **data** member of the HistoryOutput object.

        Parameters
        ----------
        frame
            A sequence of Floats specifying the frame values. **frame** can be specified in step time,
            frequency, or mode number.
        value
            A sequence of Floats specifying the value of the variable at the frame values specified
            in **frame**.

        Raises
        ------
        ValueError
            If the length of **frame** is not the same as the length of **value**.
        """
        ...

    @overload
    @abaqus_method_doc
    def addData(self, data: tuple) -> None:
        """This method adds data to the **data** member of the HistoryOutput object.

        Parameters
        ----------
        data
            A sequence of pairs of Floats specifying the pairs (*frameValue*, **value**) where
            **frameValue** is either time, frequency, or mode and **value** is the value of the
            specified variable at **frameValue**. (This value depends on the type of the variable.)
        """
        ...

    def addData(self, *args, **kwargs) -> None:
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc
from ...UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class ConstrainedSketchGeometry:
    """The ConstrainedSketchGeometry object stores the geometry of a sketch, such as lines,
    circles, arcs, and construction lines.

    .. note:: 
        This object can be accessed by::

            import sketch
            mdb.models[name].sketches[name].geometry[i]
            mdb.models[name].sketches[name].geometry[i][i]
    """

    #: An Int specifying the index of the sketch entity in the ConstrainedSketchGeometryArray.
    id: Optional[int] = None

    #: A SymbolicConstant specifying the geometry of the sketch entity. Possible values are
    #: ARC, CIRCLE, ELLIPSE, LINE, and SPLINE.
    curveType: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the type of sketch entity. Possible values are REGULAR,
    #: REFERENCE, and CONSTRUCTION.
    type: Optional[SymbolicConstant] = None

    #: A tuple of Floats specifying the **X**- and*Y*-coordinates of a point located on the
    #: geometry.
    pointOn: Optional[float] = None



from typing import overload, Optional, Tuple

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .Region import Region
from ..BasicGeometry.Cell import Cell
from ..BasicGeometry.CellArray import CellArray
from ..BasicGeometry.Edge import Edge
from ..BasicGeometry.EdgeArray import EdgeArray
from ..BasicGeometry.Face import Face
from ..BasicGeometry.FaceArray import FaceArray
from ..BasicGeometry.ReferencePoint import ReferencePoint
from ..BasicGeometry.ReferencePointArray import ReferencePointArray
from ..BasicGeometry.Vertex import Vertex
from ..BasicGeometry.VertexArray import VertexArray
from ..Mesh.MeshElement import MeshElement
from ..Mesh.MeshElementArray import MeshElementArray
from ..Mesh.MeshNode import MeshNode
from ..Mesh.MeshNodeArray import MeshNodeArray
from ..UtilityAndView.abaqusConstants import Boolean, OVERWRITE, SymbolicConstant, UNION


@abaqus_class_doc
class Set(Region):
    """If a set spans more than one part instance, the members **vertices**, **edges**, **faces**,
    **cells**, **elements**, or **nodes** return a sequence of all the queried
    vertices/edges/faces/cells/elements/nodes respectively for each part instance contained
    in the set. For example::

        assembly=mdb.models['Transmission'].rootAssembly
        clutchElements=assembly.instances['Clutch'].elements
        clutchSet=clutchElements[16:18]+clutchElements[78:80]
        housingElements=assembly.instances['Housing'].elements
        housingSet=housingElements[45:48]
        transmissionSet=assembly.Set(name='TransmissionSet', elements=(clutchSet, housingSet))
        len(transmissionSet.elements)=7
        transmissionSet.elements[0]=mdb.models['Transmission'].rootAssembly.instances['Clutch-1'].elements[16]
        transmissionSet.elements[6]=mdb.models['Transmission'].rootAssembly.instances['housing-'].elements[47]

    .. note:: 
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].allInternalSets[name]
            mdb.models[name].parts[name].allSets[name]
            mdb.models[name].parts[name].sets[name]
            import assembly
            mdb.models[name].rootAssembly.allInstances[name].sets[name]
            mdb.models[name].rootAssembly.allInternalSets[name]
            mdb.models[name].rootAssembly.allSets[name]
            mdb.models[name].rootAssembly.instances[name].sets[name]
            mdb.models[name].rootAssembly.modelInstances[i].sets[name]
            mdb.models[name].rootAssembly.sets[name]
    """

    #: A :py:class:`~abaqus.Mesh.MeshElementArray.MeshElementArray` object.
    elements: MeshElementArray = MeshElementArray([])

    #: A :py:class:`~abaqus.Mesh.MeshNodeArray.MeshNodeArray` object.
    nodes: MeshNodeArray = MeshNodeArray([])

    #: A :py:class:`~abaqus.BasicGeometry.VertexArray.VertexArray` object.
    vertices: VertexArray = VertexArray([])

    #: An :py:class:`~abaqus.BasicGeometry.EdgeArray.EdgeArray` object.
    edges: EdgeArray = EdgeArray([])

    #: A :py:class:`~abaqus.BasicGeometry.FaceArray.FaceArray` object.
    faces: FaceArray = FaceArray([])

    #: A :py:class:`~abaqus.BasicGeometry.CellArray.CellArray` object.
    cells: CellArray = CellArray([])

    #: A :py:class:`~abaqus.BasicGeometry.VertexArray.VertexArray` object.
    xVertices: VertexArray = VertexArray([])

    #: An :py:class:`~abaqus.BasicGeometry.EdgeArray.EdgeArray` object.
    xEdges: EdgeArray = EdgeArray([])

    #: A :py:class:`~abaqus.BasicGeometry.FaceArray.FaceArray` object.
    xFaces: FaceArray = FaceArray([])

    #: A :py:class:`~abaqus.BasicGeometry.ReferencePointArray.ReferencePointArray` object.
    referencePoints: ReferencePointArray = []

    @overload
    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        nodes: Tuple[MeshNode, ...] = None,
        elements: Tuple[MeshElement, ...] = None,
        region: Optional[Region] = None,
        vertices: Tuple[Vertex, ...] = None,
        edges: Tuple[Edge, ...] = None,
        faces: Tuple[Face, ...] = None,
        cells: Tuple[Cell, ...] = None,
        xVertices: Tuple[Vertex, ...] = None,
        xEdges: Tuple[Edge, ...] = None,
        xFaces: Tuple[Face, ...] = None,
        referencePoints: Tuple[ReferencePoint, ...] = (),
        skinFaces: tuple = (),
        skinEdges: tuple = (),
        stringerEdges: tuple = (),
    ):
        """This method creates a set from a sequence of objects in a model database.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].Set
                mdb.models[name].rootAssembly.Set

        Parameters
        ----------
        name
            A String specifying the repository key.
        nodes
            A sequence of MeshNode objects. The default value is None.
        elements
            A sequence of MeshElement objects. The default value is None.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying other objects to be included in the set. The default value is
            None.
        vertices
            A sequence of ConstrainedSketchVertex objects. The default value is None.
        edges
            A sequence of Edge objects. The default value is None.
        faces
            A sequence of Face objects. The default value is None.
        cells
            A sequence of Cell objects. The default value is None.
        xVertices
            A sequence of ConstrainedSketchVertex objects that excludes specific vertices from the set. The default
            value is None.
        xEdges
            A sequence of Edge objects that excludes specific edges from the set. The default value
            is None.
        xFaces
            A sequence of Face objects that excludes specific faces from the set. The default value
            is None.
        referencePoints
            A sequence of ReferencePoint objects. The default value is an empty sequence.
        skinFaces
            A tuple of tuples specifying a skin name and the sequence of faces associated with this
            skin. Valid only for geometric regions on 3D and 2D parts.
        skinEdges
            A tuple of tuples specifying a skin name and the sequence of edges associated with this
            skin. Valid only for geometric regions on Axisymmetric parts.
        stringerEdges
            A tuple of tuples specifying a stringer name and the sequence of edges associated with
            this stringer. Valid only for geometric regions on 3D and 2D parts.

        Returns
        -------
        Set
            A :py:class:`~abaqus.Region.Set.Set` object.
        """
        ...

    @overload
    @abaqus_method_doc
    def __init__(self, name: str, objectToCopy: "Set"):
        """This method copies a set from an existing set.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].Set
                mdb.models[name].rootAssembly.Set

        Parameters
        ----------
        name
            A String specifying the name of the set.
        objectToCopy
            A :py:class:`~abaqus.Region.Set.Set` object to be copied.

        Returns
        -------
        Set
            A :py:class:`~abaqus.Region.Set.Set` object.
        """
        ...

    @abaqus_method_doc
    def __init__(self, *args, **kwargs):
        ...

    def SetByBoolean(
        self, name: str, sets: Tuple["Set"], operation: SymbolicConstant = UNION
    ):
        """This method creates a set by performing a boolean operation on two or more input sets.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].Set
                mdb.models[name].rootAssembly.Set

        Parameters
        ----------
        name
            A String specifying the repository key.
        sets
            A sequence of Set objects.
        operation
            A SymbolicConstant specifying the boolean operation to perform. Possible values are
            UNION, INTERSECTION, and DIFFERENCE. The default value is UNION. Note that if DIFFERENCE
            is specified, the order of the given input sets is important; All sets specified after
            the first one are subtracted from the first one.

        Returns
        -------
        Set
            A :py:class:`~abaqus.Region.Set.Set` object.
        """
        ...

    @abaqus_method_doc
    def SetFromColor(self, name: str, color: tuple):
        """This method creates a set containing faces of the part marked with a specified color
        attribute. Third-party applications can assign color attributes to faces, and the color
        attribute can be imported into Abaqus from an ACIS file. You can use this method to
        create sets only on parts; however, you can access the sets from instances of the parts
        in the assembly.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].Set
                mdb.models[name].rootAssembly.Set

        Parameters
        ----------
        name
            A String specifying the repository key.
        color
            A sequence of three Ints specifying the RGB color. Values can range from 0 to 255. The
            first integer is for red, the second for green, and the third for blue. For example, a
            face colored in yellow is identified by:`color=(255,255,0)`

        Returns
        -------
        Set
            A :py:class:`~abaqus.Region.Set.Set` object.
        """
        ...

    @abaqus_method_doc
    def SetFromElementLabels(self, name: str, elementLabels: tuple):
        """This method creates a set from a sequence of element labels in a model database.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].Set
                mdb.models[name].rootAssembly.Set

        Parameters
        ----------
        name
            A String specifying the repository key.
        elementLabels
            A sequence of element labels. An element label is a sequence of Int element identifiers.
            For example, for a part:`elementLabels=(2,3,5,7)`For an
            assembly:`elementLabels=(('Instance-1', (2,3,5,7)),       ('Instance-2', (1,2,3)))`

        Returns
        -------
        Set
            A :py:class:`~abaqus.Region.Set.Set` object.
        """
        ...

    @abaqus_method_doc
    def SetFromNodeLabels(self, name: str, nodeLabels: tuple, unsorted: Boolean = False):
        """This method creates a set from a sequence of node labels in a model database.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].Set
                mdb.models[name].rootAssembly.Set

        Parameters
        ----------
        name
            A String specifying the repository key.
        nodeLabels
            A sequence of node labels. A node label is a sequence of Int node identifiers. For
            example, for a part:`nodeLabels=(2,3,5,7)`For an assembly:`nodeLabels=(('Instance-1',
            (2,3,5,7)), ('Instance-2', (1,2,3)))`
        unsorted
            A Boolean specifying whether the created set is unsorted. The default value is False.

            .. versionadded:: 2018
                The `unsorted` argument was added.

        Returns
        -------
        Set
            A :py:class:`~abaqus.Region.Set.Set` object.
        """
        ...

    @abaqus_method_doc
    def MapSetsFromOdb(
        self, odbPath: str, odbSets: str, partSets: str = "", method: str = OVERWRITE
    ):
        """This method creates sets based on mapping sets from element centroid locations in an
        Odb.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].Set
                mdb.models[name].rootAssembly.Set

        Parameters
        ----------
        odbPath
            A String specifying the path to the ODB containing the source sets.
        odbSets
            A list of names of sets on the ODB to map.
        partSets
            A list of names of sets to construct or use corresponding to the ODB sets.
        method
            An enum to specify OVERWRITE, APPEND, INTERSECT, or REMOVE. The default is OVERWRITE.

        Returns
        -------
        Set
            A :py:class:`~abaqus.Region.Set.Set` object or a tuple of Set objects.
        """
        ...



from __future__ import annotations

from typing import Union, Optional, Tuple, overload

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .Region import Region
from ..BasicGeometry.EdgeArray import EdgeArray
from ..BasicGeometry.Face import Face
from ..BasicGeometry.FaceArray import FaceArray
from ..Mesh.MeshElementArray import MeshElementArray
from ..Mesh.MeshNodeArray import MeshNodeArray
from ..UtilityAndView.SymbolicConstant import abaqusConstants as C
from ..UtilityAndView.abaqusConstants import SymbolicConstant, UNION


@abaqus_class_doc
class Surface(Region):
    """The Surface object stores surfaces selected from the assembly. A surface is comprised of
    geometric or discrete entities but not both. An instance of a Surface object is
    available from the **surface** member of the Assembly object.

    .. note:: 
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].allInternalSurfaces[name]
            mdb.models[name].parts[name].allSurfaces[name]
            mdb.models[name].parts[name].surfaces[name]
            import assembly
            mdb.models[name].rootAssembly.allInstances[name].surfaces[name]
            mdb.models[name].rootAssembly.allInternalSurfaces[name]
            mdb.models[name].rootAssembly.allSurfaces[name]
            mdb.models[name].rootAssembly.instances[name].surfaces[name]
            mdb.models[name].rootAssembly.modelInstances[i].surfaces[name]
            mdb.models[name].rootAssembly.surfaces[name]
    """

    #: An :py:class:`~abaqus.BasicGeometry.EdgeArray.EdgeArray` object.
    edges: EdgeArray = EdgeArray([])

    #: A :py:class:`~abaqus.BasicGeometry.FaceArray.FaceArray` object.
    faces: FaceArray = FaceArray([])

    #: A :py:class:`~abaqus.Mesh.MeshElementArray.MeshElementArray` object.
    elements: MeshElementArray = MeshElementArray([])

    #: A :py:class:`~abaqus.Mesh.MeshNodeArray.MeshNodeArray` object.
    nodes: MeshNodeArray = MeshNodeArray([])

    #: A tuple of SymbolicConstants specifying the sides; for example, (SIDE1, SIDE2).
    sides: Optional[Tuple[SymbolicConstant, ...]] = None

    #: A tuple of Ints specifying the instances. This member is not applicable for a Surface
    #: object on an output database.
    instances: Optional[int] = None

    @overload
    @abaqus_method_doc
    def __init__(
        self,
        side1Faces: Union[Face, Tuple[Face, ...], None] = None,
        side2Faces: Union[Face, Tuple[Face, ...], None] = None,
        side12Faces: Union[Face, Tuple[Face, ...], None] = None,
        end1Edges: Union[Face, Tuple[Face, ...], None] = None,
        end2Edges: Union[Face, Tuple[Face, ...], None] = None,
        circumEdges: Union[Face, Tuple[Face, ...], None] = None,
        side1Edges: Union[Face, Tuple[Face, ...], None] = None,
        side2Edges: Union[Face, Tuple[Face, ...], None] = None,
        face1Elements: Union[Face, Tuple[Face, ...], None] = None,
        face2Elements: Union[Face, Tuple[Face, ...], None] = None,
        face3Elements: Union[Face, Tuple[Face, ...], None] = None,
        face4Elements: Union[Face, Tuple[Face, ...], None] = None,
        face5Elements: Union[Face, Tuple[Face, ...], None] = None,
        face6Elements: Union[Face, Tuple[Face, ...], None] = None,
        side1Elements: Union[Face, Tuple[Face, ...], None] = None,
        side2Elements: Union[Face, Tuple[Face, ...], None] = None,
        side12Elements: Union[Face, Tuple[Face, ...], None] = None,
        end1Elements: Union[Face, Tuple[Face, ...], None] = None,
        end2Elements: Union[Face, Tuple[Face, ...], None] = None,
        circumElements: Union[Face, Tuple[Face, ...], None] = None,
        name: str = "",
        **kwargs
    ) -> None:
        """This method creates a surface from a sequence of objects in a model database. The
        surface will apply to the sides specified by the arguments.For example
        surface=mdb.models['Model-1'].parts['Part-1'].Surface(side1Faces=side1Faces,
        name='Surf-1')

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].Surface
                mdb.models[name].rootAssembly.Surface

        Parameters
        ----------
        name
            A String specifying the repository key. The default value is an empty string.
        side1Elements
            A sequence of MeshElement objects (surface applies to SIDE1 of element). 
            The default value is None.
        side2Elements
            A sequence of MeshElement objects (surface applies to SIDE2 of element). 
            The default value is None.
        side12Elements
            A sequence of MeshElement objects (surface applies to both SIDE1 and SIDE2 of element). 
            The default value is None.
        end1Elements
            A sequence of MeshElement objects (surface applies to END1 of element). 
            The default value is None.
        end2Elements
            A sequence of MeshElement objects (surface applies to END2 of element). 
            The default value is None.
        circumElements
            A sequence of MeshElement objects (surface applies to circumference of element). 
            The default value is None.
        face1Elements
            A sequence of MeshElement objects (surface applies to FACE1 of element) or MeshFace objects. 
            The default value is None.
        face2Elements
            A sequence of MeshElement objects (surface applies to FACE2 of element) or MeshFace objects. 
            The default value is None.
        face3Elements
            A sequence of MeshElement objects (surface applies to FACE3 of element) or MeshFace objects. 
            The default value is None.
        face4Elements
            A sequence of MeshElement objects (surface applies to FACE4 of element) or MeshFace objects. 
            The default value is None.
        face5Elements
            A sequence of MeshElement objects (surface applies to FACE5 of element) or MeshFace objects. 
            The default value is None.
        face6Elements
            A sequence of MeshElement objects (surface applies to FACE6 of element) or MeshFace objects. 
            The default value is None.
        side1Faces
            A sequence of Face objects (surface applies to SIDE1 of face). The default value is None.
        side2Faces
            A sequence of Face objects (surface applies to SIDE2 of face). The default value is None.
        side12Faces
            A sequence of Face objects (surface applies to both SIDE1 and SIDE2 of face). 
            The default value is None.
        side1Edges
            A sequence of Edge objects (surface applies to SIDE1 of edge). The default value is None.
        side2Edges
            A sequence of Edge objects (surface applies to SIDE2 of edge). The default value is None.
        end1Edges
            A sequence of Edge objects (surface applies to END1 of edge). The default value is None.
        end2Edges
            A sequence of Edge objects (surface applies to END2 of edge). The default value is None.
        circumEdges
            A sequence of Edge objects (surface applies circumferentially to edge). 
            The default value is None.
        kwargs
            The required parameters for different conditions are:
            
            - On three-dimensional solid faces, you can use the following arguments: 
              side1Faces, side2Faces

            - On three-dimensional shell faces, you can use the following arguments: 
              side1Faces, side2Faces, side12Faces

            - On three-dimensional wire edges, you can use the following arguments:
              end1Edges, end2Edges, circumEdges

            - On three-dimensional or two-dimensional or axisymmetric edges, you can use the following arguments:
              side1Edges, side2Edges

            - On two-dimensional or axisymmetric shell elements, you can use the following arguments:
              face1Elements, face2Elements, face3Elements, face4Elements

            - On solid elements, you can use the following arguments:
              face1Elements, face2Elements, face3Elements, face4Elements, face5Elements, face6Elements

            - On three-dimensional shell elements, you can use the following arguments:
              side1Elements, side2Elements, side12Elements

            - On three-dimensional wire elements, you can use the following arguments:
              end1Elements, end2Elements, circumElements

            - On two-dimensional or axisymmetric wire elements, you can use the following arguments:
              side1Elements, side2Elements

        Returns
        -------
        Surface
            A :py:class:`~abaqus.Region.Surface.Surface` object.
        """
        ...

    @overload
    @abaqus_method_doc
    def __init__(self, name: str, objectToCopy: Surface) -> None:
        """This method copies a surface from an existing surface.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].Surface
                mdb.models[name].rootAssembly.Surface

        Parameters
        ----------
        name
            A String specifying the name of the surface.
        objectToCopy
            A :py:class:`~abaqus.Region.Surface.Surface` object to be copied.

        Returns
        -------
        Surface
            A :py:class:`~abaqus.Region.Surface.Surface` object.
        """
        ...

    @abaqus_method_doc
    def __init__(self, *args, **kwargs) -> None:
        ...

    def SurfaceByBoolean(
        self,
        name: str,
        surfaces: Tuple[Surface],
        operation: Literal[C.UNION, C.INTERSECTION, C.DIFFERENCE] = UNION,
    ) -> Surface:
        """This method creates a surface by performing a boolean operation on two or more input
        surfaces.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].Surface
                mdb.models[name].rootAssembly.Surface

        Parameters
        ----------
        name
            A String specifying the repository key.
        surfaces
            A sequence of Surface objects.
        operation
            A SymbolicConstant specifying the boolean operation to perform. Possible values are
            UNION, INTERSECTION, and DIFFERENCE. The default value is UNION. Note that if DIFFERENCE
            is specified, the order of the given input surfaces is important; All surfaces specified
            after the first one are subtracted from the first one.

        Returns
        -------
        Surface
            A :py:class:`~abaqus.Region.Surface.Surface` object.
        """
        ...

    @abaqus_method_doc
    def SurfaceFromElsets(self, name: str, elementSetSeq: tuple) -> Surface:
        """This method creates a surface from a sequence of element sets in a model database.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].Surface
                mdb.models[name].rootAssembly.Surface

        Parameters
        ----------
        name
            A String specifying the repository key.
        elementSetSeq
            A sequence of element sets. For example,`elementSetSeq=((elset1, S1),(elset2, S2))`where
            `elset1=mdb.models[name].rootAssembly.sets['Clutch']` and `S1` and `S2` indicate the
            side of the element set.

        Returns
        -------
        Surface
            A :py:class:`~abaqus.Region.Surface.Surface` object.
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import Boolean


@abaqus_class_doc
class InputFile:
    """The InputFile object is used to store the definitions in an Abaqus input file. InputFile
    objects can be created using the methods described in this section.

    .. note:: 
        This object can be accessed by::

            import inpParser
    """

    #: A String specifying the source file name of the Abaqus input file.
    file: str = ""

    #: A String specifying the directory where the input file is located.
    directory: str = ""

    #: A sequence of Strings specifying any additional input files included in the specified
    #: input file.
    includes: tuple = ()

    #: A sequence of Strings for input files included in the specified input file that could
    #: not be located.
    missingIncludes: tuple = ()

    @abaqus_method_doc
    def __init__(self, file: str, directory: str = ""):
        """This method creates an InputFile object by reading an Abaqus input file.

        .. note:: 
            This function can be accessed by::

                inpParser.InputFile

        Parameters
        ----------
        file
            A String specifying the path to the input file.
        directory
            A String specifying the path to the directory containing the input file.

        Returns
        -------
        InputFile
            An :py:class:`~abaqus.InputFileParser.InputFile.InputFile` object.
        """
        ...

    @abaqus_method_doc
    def parse(
        self,
        organize: Boolean = False,
        verbose: Boolean = False,
        bulk: Boolean = True,
        usePyArray: Boolean = False,
    ):
        """This method parses the input file associated with the InputFile object.

        Parameters
        ----------
        organize
            A Boolean specifying whether keywords should be organized into suboptions. The default
            is False.
        verbose
            A Boolean specifying whether verbose output is to be printed. If **verbose** is True,
            information about fatal errors is printed. If no fatal errors occur, there is no output.
            The default is False.
        bulk
            A Boolean specifying whether the input file includes bulk data that should be parsed.
            The default is True.
        usePyArray
            A Boolean specifying that parse method can return an AbaqusNDarray object for a keyword
            data value. In cases where large amounts of numerical data (i.e., large node arrays) are
            expected, it is recommended that you use the option usePyArray=True. The default is
            False.

        Returns
        -------
        KeywordSequence
            A KeywordSequence object.

        Raises
        ------
        ValueError
            If you parse an input file more than once, a ValueError is raised for each subsequent
            parsing.
        """
        ...
